/*****************************************************
Chip type           : ATmega8515
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256
*****************************************************/

#include <mega8515.h> 
#include <delay.h>
#include <stdlib.h>
#include <stdio.h>
#include <avr_eeprom.c>
#include <i2c.h>
#include <lcd.h>

//I2C Initialization
#asm
   .equ __i2c_port=0x18 ;PORTB
   .equ __sda_bit=2
   .equ __scl_bit=3
#endasm

// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x15 ;PORTC
#endasm

//************************************************************
//   RC5
//************************************************************
unsigned char cnt_bit = 0;              //  
unsigned char code [14];                // 
unsigned char counter = 0;              //"" 

#define l_min    40                     //  "" (long)   ,  "" , 1,28..2,176 
#define l_max    68
#define s_min    22                     //  "" (short)   ,  "" , 0,704..1,088
#define s_max    34

bit state =  0;                                 //   
bit error = 0;                                  // 

//  ,  14 
unsigned char startbit1 = 0;                    //  , .. = 1
unsigned char startbit2 = 0;                    //  , .. = 1                    
unsigned char toggle = 0;                       // ,      ,   -  .!
unsigned char address = 0;                      // , 5 
unsigned char command = 0;                      // (), 6 

//-  
const unsigned char RC5address = 0x15;          //
const unsigned char RC5cmdpower = 0x01;         // -  Power
const unsigned char RC5cmdred = 0x14;           // 
const unsigned char RC5cmdgreen = 0x15;         // 
const unsigned char RC5cmdyellow = 0x16;        // 
const unsigned char RC5cmdblue = 0x17;          // 
const unsigned char RC5cmdmenu = 0x0F;          //   -  
const unsigned char RC5cmdmute = 0x18;          //mute -  mute
const unsigned char RC5cmdup1 = 0x25;           // (+) -  Vol+
const unsigned char RC5cmdup2 = 0x0D;           // (+) -  Up
const unsigned char RC5cmddown1 = 0x26;         // (-) -  Vol-
const unsigned char RC5cmddown2 = 0x11;         // (-) -  Down
const unsigned char RC5cmdnext1 = 0x23;         // > -  Ch+
const unsigned char RC5cmdnext2 = 0x10;         // > -  >
const unsigned char RC5cmdback1 = 0x24;         // < -  Ch-
const unsigned char RC5cmdback2 = 0x0E;         // < -  < 
const unsigned char RC5cmddig1 = 0x03;          // 1
const unsigned char RC5cmddig2 = 0x04;          // 2
const unsigned char RC5cmddig3 = 0x05;          // 3

unsigned char colorbutton = 0;                  //   
#define colour  RC5cmdred                       //      ?

// External Interrupt 0 service routine
interrupt [EXT_INT0] void ext_int0_isr(void)
        {
        counter = TCNT0;                        //  -
        TCNT0 = 0x00;                           // 
        error = 1;                              //  -    ,   
               
        if (cnt_bit == 0)                       //    
                {
                TCCR0 = 0x04;                           // ,  31250 (      )
                code [cnt_bit] = !PIND.2;               //  , PIND.2 -  ,  0
                cnt_bit++;                              //   
                state = 1;                              // , ..  1
                }
                  
                else                                        //   (   )
                        {
                        if ((counter>s_min) && (counter<s_max)) //  
                                {
                                if (state)                      //  1
                                        {
                                        state = 0;              // 
                                        error = 0;              // , ..  
                                        }
                                        else                    //  0
                                                {
                                                state = 1;                      // , ..  - 1
                                                code[cnt_bit] = !PIND.2;        //  -  
                                                cnt_bit++;                      //  
                                                error=0;                        // , ..  
                                                };
                                };     
                   
                        if ((counter>l_min) && (counter<l_max)) //  
                                {
                                code[cnt_bit] = !PIND.2;        //  -  
                                cnt_bit++;                      //  
                                error = 0;                      // , ..  
                                };
                                                  
                        if (error == 1)                         //       ,  
                                {
                                char i = 0;                     // 
                                
                                TCCR0 = 0x00;                   // 
                                TCNT0 = 0x00;                   // 
                                counter = 0;                    // 
                                cnt_bit = 0;                    //   
                                for (i=0; i<14; i++) code[i]=0; //   
                                };
                   
                        if (cnt_bit == 14)                      //   14 
                                {
                                char i = 0;                     // 
                                TCCR0 = 0x00;                   // 
                                TCNT0 = 0x00;;                  // 
                                cnt_bit = 0;                    //   
                                
                                startbit1 = code[0];            //  
                                startbit2 = code[1];            //  
                                toggle = code[2];               //-
                                                                //    
                                address = (code[3] << 4)|(code[4] << 3)|(code[5] << 2)|(code[6] << 1)|code[7];                    
                                command = (code[8] << 5)|(code[9] << 4)|(code[10] << 3 )|(code[11] << 2)|(code[12] << 1)|code[13];
                                     
                                for (i=0; i<14; i++) code[i]=0; //   
                                
                                if (address == RC5address && (command == RC5cmdred || command == RC5cmdgreen || command == RC5cmdblue || command == RC5cmdyellow)) colorbutton = command; 
                                };
                        };                                                                                
        }

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
        {
        char i = 0;                     // 
        
        TCCR0 = 0x00;                   // 
        TCNT0 = 0x00;                   // 
        counter = 0;                    // 
        cnt_bit = 0;                    //   
        for (i=0; i<14; i++) code[i]=0; //   
        }
//************************************************************

//************************************************************
//     
//************************************************************
const unsigned char tda_addr = 0b10001000;      //I2C  

//    
//0., 2..3 <-> 2..1 (    2    )
//1. - 0, (0..15)<->(0..30),  2
//2. - 0, (0..7)<->(0..-7), (8..40)<->(-8..-40), 56 - MUTE
//3.bass (0..7)<->(-14..0), (15..8)<->(0..14)dB
//4.treble (0..7)<->(-14..0), (15..8)<->(0..14)dB
//5.  - 0, (0..7)<->(0..-7), (8..72)<->(-8..-72), 127 - MUTE
//6.  - 0, (0..7)<->(0..-7), (8..72)<->(-8..-72), 127 - MUTE

//                                0.input    1.gain     2.volume   3.bass     4.treble   5.right    6.left
unsigned char parameter[7][3] = {{3, 2, 2}, {0, 0, 0}, {0, 0, 0}, {7, 7, 7}, {7, 7, 7}, {0, 0, 0}, {0, 0, 0}};          //3 - - , 7 - -  ,    
const unsigned char sub_address[7] = {0, 1, 2, 4, 5, 6, 7};                                                             //           

unsigned char change_flags[7][3] = {{0, 0, 0}, {0, 0, 0}, {0, 0, 0}, {0, 0, 0}, {0, 0, 0}, {0, 0, 0}, {0, 0, 0}};       //  

//       load() (      parameter[7][3])
const unsigned char gain = 1;
const unsigned char volume = 2;
const unsigned char bass = 3;
const unsigned char treble = 4;
const unsigned char right = 5;
const unsigned char left = 6;

unsigned char bass_index[3] = {0, 0, 0};        //        (     ,     )      
unsigned char treble_index[3] = {0, 0, 0};
unsigned char input = 0;                        // 
bit input_ch = 0;                               //,     

void load (unsigned char param)                 //    
        {
        #asm ("cli")                            // 
        i2c_start ();
        i2c_write (tda_addr);                   //    
        i2c_write (sub_address[param]);          
        i2c_write (parameter[param][input]);                
        i2c_stop ();
        change_flags[param][input] = 1;         //   
        #asm ("sei")                            // 
        }
//************************************************************

//************************************************************
//   
//************************************************************
unsigned char buf1[16];         //   LCD -   
unsigned char buf2[16];         //   LCD -   
unsigned char pbuf1[16];        //   LCD -   
unsigned char pbuf2[16];        //   LCD -   

unsigned char x = 0;            //       
unsigned char y = 0;

bit power = 0;                  //,  =1 - 
bit power_cmplt = 0;            //,       (. - main)
bit menu = 0;                   //,  =1 -    
bit mute = 0;                   //,  =1 - mute 
bit mute_cmplt = 0;             //,       mute (. - main)
unsigned char menu_index = 0;   //  
unsigned char backlight = 0;    // 
bit backlight_ch = 0;           //   
bit beep = 0;                   //  
bit beep_ch = 0;                //   

//     
const unsigned char input_e = 2;        //       
const unsigned char backlight_e = 3;    //    
const unsigned char beep_e = 4;         //    

#define blight          PORTC.3         //   
#define add_input       PORTB.6         //       
#define snd             PORTB.0         //   
#define en              1               //          
#define dis             0               //

#define col1            PORTA.3         //  1
#define col2            PORTA.2         //  2
#define col3            PORTA.1         //  3
#define col4            PORTA.0         //  4
#define row1            !PINA.6         //  1
#define row2            !PINA.5         //  2
#define row3            !PINA.4         //  3
 
unsigned char but[12];                  //     
unsigned char b[12];                    //     (. main)
unsigned char summbut;                  //     

#define powerb          0               //     
#define menub           6
#define muteb           8
#define upb             2
#define downb           10
#define nextb           5
#define prevb           7
                
// Timer 1 overflow interrupt service routine
interrupt [TIM1_OVF] void timer1_ovf_isr(void)          // 6.5       .
        {
        unsigned char m, n;
        
        for (m=0; m<7; m++)                             //         
                {
                for (n=0; n<3; n++)
                        {
                        if (change_flags[m][n]) writeEEPROM (3*m + n + 5, parameter[m][n]);     //   -    (5 -     )
                        change_flags[m][n] = 0;                                                 //  
                        }
                }
        if (backlight_ch) writeEEPROM (backlight_e, backlight);         //   
        if (input_ch) writeEEPROM (input_e, input);                     //    
        if (beep_ch) writeEEPROM (beep_e, beep);                        //   
        }

void sound (int time)            
        {
        int i = 0;
        while (i < 4*time)
                {  
                snd = 1;
                delay_us(150);
                snd = 0;
                delay_us(150);                
                i++;
                }                                                                                                                                                                                                                                                                                     
        }
//------------------------------------------------------------

void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port A initialization
// Func7=In Func6=In Func5=In Func4=In Func3=Out Func2=Out Func1=Out Func0=Out 
// State7=T State6=P State5=P State4=P State3=1 State2=1 State1=1 State0=1 
PORTA=0x7F;
DDRA=0x0F;

// Port B initialization
// Func7=In Func6=Out Func5=In Func4=In Func3=Out Func2=Out Func1=In Func0=Out 
// State7=T State6=0 State5=T State4=T State3=0 State2=0 State1=T State0=0 
PORTB=0x00;
DDRB=0x4D;

// Port C initialization
// Func7=In Func6=In Func5=In Func4=In Func3=Out Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=0 State2=T State1=T State0=T 
PORTC=0x00;
DDRC=0x08;

// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=P State1=T State0=T 
PORTD=0x04;
DDRD=0x00;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Normal top=FFh
// OC0 output: Disconnected
TCCR0=0x00;
TCNT0=0x00;
OCR0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 7,813 kHz
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: On
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x05;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Any Change
// INT1: Off
// INT2: Off
GICR|=0x40;
MCUCR=0x01;
MCUCSR=0x00;
GIFR=0x40;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x82;


// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;

        for (x=0; x<7; x++)                                                     //          
                {
                for (y=0; y<3; y++)
                        {
                        unsigned char temp_data = 0;
                        temp_data = readEEPROM (3*x + y + 5);
                        if (temp_data <= 127) parameter[x][y] = temp_data;      //    
                                else writeEEPROM (3*x + y + 5, parameter[x][y]);//   ,    (       ) 
                        }
                }
        
        if (readEEPROM (input_e) > 2) input = 0;                                //  ,   
                else input = readEEPROM (input_e);
        if (readEEPROM (backlight_e) > 10) backlight = 0;
                else backlight = readEEPROM (backlight_e);
        if (readEEPROM (beep_e) > 1) beep = 0;
                else beep = readEEPROM (beep_e);

        #asm ("sei")
        
        delay_ms(200);
        
        lcd_init(16);
        i2c_init();
        
while (1)
        {
        for (x = 0; x < 25; x++)                //  
                {
                delay_ms (6);                   //24   6
                col1 = 0;                       //    
                delay_ms (1);
                if (row1) b[0]++;               //  = 0,       
                if (row2) b[4]++;
                if (row3) b[8]++;
                col1 = 1;                       //   1,   
                col2 = 0;
                delay_ms (1);
                if (row1) b[1]++;
                if (row2) b[5]++;
                if (row3) b[9]++;
                col2 = 1;
                col3 = 0;
                delay_ms (1);
                if (row1) b[2]++;
                if (row2) b[6]++;
                if (row3) b[10]++;
                col3 = 1;
                col4 = 0;
                delay_ms (1);
                if (row1) b[3]++;
		if (row2) b[7]++;
                if (row3) b[11]++;
                col4 = 1;
                }
        summbut = 0;                            //     
        for (x = 0; x < 11; x++)                //  12- 
                {
                if (b[x] >= 6)                  //    6 
                        {
                        but[x] = 1;             //,    
                        summbut++;              // 
                        }
                        else but[x] = 0;        // -   
                b[x] = 0;                       //  
                }
        
        if (summbut > 1) for (x = 0; x < 11; x++) but[x] = 0;   //     1  -  
        
        if (address == RC5address && command == RC5cmdpower && !power)  //         
                {
                power = 1;                                              // 
                address = command = 0;                                  //  
                if (beep) sound(30);                                    //   - 
                delay_ms(150);
                }
        if (address == RC5address && command == RC5cmdpower && power && colorbutton == colour)  //     ,       
                {
                power = 0;                                                                      // 
                address = command = 0;                                                          //  
                if (beep) sound(30);                                                            //   - 
                delay_ms(150);
                }
                
        if (but[powerb])                //    
                {
                power = !power;         /// 
                delay_ms(150);
                }
        
        if (power)                      //  
                { 
                if (power_cmplt)        //  ,    
                        {
                        power_cmplt = 0;                        //  
                        for (x=0; x<7; x++) load (x);           //           
                        if (input == 2) add_input = en;         //   2 -  
                                else add_input = dis;           //  -  
                        lcd_gotoxy(0, 0);                       //   LCD
                        lcd_puts(buf1);
                        lcd_gotoxy(0, 1);
                        lcd_puts(buf2);
                        }
        
                if (backlight) blight = en;     //   -  
                        else blight = dis;
        
                if (but[menub] || (address == RC5address && command == RC5cmdmenu && colorbutton == colour))    ///   
                        {
                        menu = !menu;                                                                           // 
                        address = command = 0;
                        if (beep) sound(30);                                                                    
                        if (menu) buf2[0] = 0;                                                                  //   -   
                        delay_ms(50);
                        }
                
                if (but[muteb] || (address == RC5address && command == RC5cmdmute && colorbutton == colour))    /// -  
                        {
                        mute = !mute;                                                                           // 
                        address = command = 0;
                        if (beep) sound(30);
                        delay_ms(50);
                        }
                
                if (mute)                                       //  
                        {
                        i2c_start ();                           // .   2
                        i2c_write (tda_addr);
                        i2c_write (sub_address[volume]);          
                        i2c_write (56);                
                        i2c_stop ();
                        mute_cmplt = 1;                         // ,   
                        }
                        else if (mute_cmplt)                    //      ,   
                                {
                                mute_cmplt = 0;                 // 
                                load (volume);                  //  
                                }
        
                if (!menu)                                              //    
                        {
                        if (address == RC5address && colorbutton == colour && (command == RC5cmddig1 || command == RC5cmddig2 || command == RC5cmddig3))
                                {                                       //     
                                if (command == RC5cmddig1) input = 0;   // . 
                                if (command == RC5cmddig2) input = 1;  
                                if (command == RC5cmddig3) input = 2;
                                for (x=0; x<7; x++) load (x);           //    
                                if (input == 2) add_input = en;         // ,    2
                                        else add_input = dis;
                                input_ch = 1;                           //  ,   
                                delay_ms(50);
                                }
                                
                        if (but[nextb] || (address == RC5address && colorbutton == colour && (command == RC5cmdnext1 || command == RC5cmdnext2))) 
                                {                                       //   "" ( )
                                if (++input > 2) input = 0;
                                for (x=0; x<7; x++) load (x);
                                if (input == 2) add_input = en;
                                        else add_input = dis;
                                input_ch = 1;
                                delay_ms(50);
                                }
                
                        if (but[prevb] || (address == RC5address && colorbutton == colour && (command == RC5cmdback1 || command == RC5cmdback2))) 
                                {                                       //   "" ( )   
                                if (--input > 2) input = 2;
                                for (x=0; x<7; x++) load (x);
                                if (input == 2) add_input = en;
                                        else add_input = dis;
                                input_ch = 1;
                                delay_ms(50);
                                }        
                
                        if ((but[downb] || (address == RC5address && colorbutton == colour && (command == RC5cmddown1 || command == RC5cmddown2))) && parameter[volume][input] < 56)
                                {                                                                       //        ""
                                if (parameter[volume][input] < 40) parameter[volume][input]++;          //  
                                        else parameter[volume][input] = parameter[volume][input] + 16;
                                load (volume);                                                          // 
                                }
                        if ((but[upb] || (address == RC5address && colorbutton == colour && (command == RC5cmdup1 || command == RC5cmdup2))) && parameter[volume][input] > 0)
                                {                                                                       //        ""
                                if (parameter[volume][input] < 41) parameter[volume][input]--;
                                        else parameter[volume][input] = parameter[volume][input] - 16;
                                load (volume);
                                }
        
                        if (mute) sprintf (buf1, "Input %u - MUTE", input+1);                            //   -  
                                else sprintf (buf1, "Input %u", input+1); 
                
                        if (parameter[volume][input] && parameter[volume][input] < 56) sprintf (buf2, "Volume -%udB", parameter[volume][input]);       //  -  
                                else if (parameter[volume][input] == 56) sprintf (buf2, "MUTE");                                                       //    
                                        else sprintf (buf2, "Volume 0dB");
                        }
                
                if (menu)            //   
                        {            //  ""/""    
                        if ((but[nextb] || (address == RC5address && colorbutton == colour && (command == RC5cmdnext1 || command == RC5cmdnext2))) && ++menu_index > 6) menu_index = 0; 
                        if ((but[prevb] || (address == RC5address && colorbutton == colour && (command == RC5cmdback1 || command == RC5cmdback2))) && --menu_index > 6) menu_index = 6; 
                        
                        if (but[nextb] || but[prevb]) delay_ms(50);
                        
                        switch (menu_index)     //       ""/"" . .
                                {               //     
                                case 0:         //gain
                                        {
                                        if ((but[upb] || (address == RC5address && colorbutton == colour && (command == RC5cmdup1 || command == RC5cmdup2))) && parameter[gain][input] < 14)
                                                {
                                                parameter[gain][input] = parameter[gain][input] + 2;
                                                load (gain);
                                                }
                                        if ((but[downb] || (address == RC5address && colorbutton == colour && (command == RC5cmddown1 || command == RC5cmddown2))) && parameter[gain][input] > 0)
                                                {
                                                parameter[gain][input] = parameter[gain][input] - 2;
                                                load (gain);
                                                }
                                        sprintf (buf2, " Gain %udB        ", parameter[gain][input]);
                                        }; break;
                                case 1:         //bass
                                        {
                                        if (parameter[bass][input] < 8) bass_index[input] = parameter[bass][input];
                                                else bass_index[input] = 23 - parameter[bass][input];  
                                
                                        if ((but[upb] || (address == RC5address && colorbutton == colour && (command == RC5cmdup1 || command == RC5cmdup2))) && bass_index[input] < 15) 
                                                {
                                                bass_index[input]++;
                                                load (bass);
                                                }
                                        if ((but[downb] || (address == RC5address && colorbutton == colour && (command == RC5cmddown1 || command == RC5cmddown2))) && bass_index[input] > 0) 
                                                {
                                                bass_index[input]--;
                                                load (bass);
                                                }
                                        if (bass_index[input] < 8) parameter[bass][input] = bass_index[input];
                                                else parameter[bass][input] = 23 - bass_index[input];
                                
                                        if (bass_index[input] == 7 || bass_index[input] == 8) sprintf (buf2, " Bass 0dB        ");
                                        if (bass_index[input] < 7) sprintf (buf2, " Bass -%udB        ", 14 - 2*bass_index[input]);
                                        if (bass_index[input] > 8) sprintf (buf2, " Bass +%udB        ", (bass_index[input] - 8)*2);
                                        }; break;
                                case 2:         //treble
                                        {
                                        if (parameter[treble][input] < 8) treble_index[input] = parameter[treble][input];
                                                else treble_index[input] = 23 - parameter[treble][input];  
                                
                                        if ((but[upb] || (address == RC5address && colorbutton == colour && (command == RC5cmdup1 || command == RC5cmdup2))) && treble_index[input] < 15) 
                                                {
                                                treble_index[input]++; 
                                                load (treble);
                                                }
                                        if ((but[downb] || (address == RC5address && colorbutton == colour && (command == RC5cmddown1 || command == RC5cmddown2))) && treble_index[input] > 0) 
                                                {
                                                treble_index[input]--; 
                                                load (treble);
                                                }
                                        
                                        if (treble_index[input] < 8) parameter[treble][input] = treble_index[input];
                                                else parameter[treble][input] = 23 - treble_index[input];
                                
                                        if (treble_index[input] == 7 || treble_index[input] == 8) sprintf (buf2, " Treble 0dB        ");
                                        if (treble_index[input] < 7) sprintf (buf2, " Treble -%udB        ", 14 - 2*treble_index[input]);
                                        if (treble_index[input] > 8) sprintf (buf2, " Treble +%udB        ", (treble_index[input] - 8)*2);
                                        }; break;
                                case 3:         //right
                                        {
                                        if ((but[downb] || (address == RC5address && colorbutton == colour && (command == RC5cmddown1 || command == RC5cmddown2))) && parameter[right][input] < 72)
                                                {
                                                parameter[right][input]++;
                                                load (right);
                                                }
                                        if ((but[upb] || (address == RC5address && colorbutton == colour && (command == RC5cmdup1 || command == RC5cmdup2))) && parameter[right][input] > 0)
                                                {
                                                parameter[right][input]--;
                                                load (right);
                                                }
                                        if (parameter[right][input]) sprintf (buf2, " Right -%udB        ", parameter[right][input]);
                                                else sprintf (buf2, " Right 0dB        "); 
                                        }; break;
                                case 4:         //left
                                        {
                                        if ((but[downb] || (address == RC5address && colorbutton == colour && (command == RC5cmddown1 || command == RC5cmddown2))) && parameter[left][input] < 72)
                                                {
                                                parameter[left][input]++;
                                                load (left);
                                                }
                                        if ((but[upb] || (address == RC5address && colorbutton == colour && (command == RC5cmdup1 || command == RC5cmdup2))) && parameter[left][input] > 0)
                                                {
                                                parameter[left][input]--;
                                                load (left);
                                                }
                                        if (parameter[left][input]) sprintf (buf2, " Left -%udB        ", parameter[left][input]);
                                                else sprintf (buf2, " Left 0dB        ");
                                        }; break;
                                case 5:         //backlight
                                        {
                                        if ((but[upb] || (address == RC5address && colorbutton == colour && (command == RC5cmdup1 || command == RC5cmdup2))) || (but[downb] || (address == RC5address && (command == RC5cmddown1 || command == RC5cmddown2))))
                                                {
                                                backlight = !backlight;
                                                backlight_ch = 1;
                                                }
                                        if (backlight) sprintf (buf2, " Backlight on        ");
                                                else sprintf (buf2, " Backlight off        ");
                                        delay_ms(50);
                                        }; break;
                                case 6:         //beep
                                        {
                                        if ((but[upb] || (address == RC5address && colorbutton == colour && (command == RC5cmdup1 || command == RC5cmdup2))) || (but[downb] || (address == RC5address && (command == RC5cmddown1 || command == RC5cmddown2))))
                                                {
                                                beep = !beep;
                                                beep_ch = 1;
                                                }
                                        if (beep) sprintf (buf2, " Beep on        ");
                                                else sprintf (buf2, " Beep off        ");
                                        delay_ms(50);
                                        };       
                                }       
                
                        sprintf (buf1, "Input %u", input+1);
                        buf2[0] = '<';          //    
                        buf2[15] = '>';
                        }
                for (x = 0; x < 16; x++)                                        //  16-  
                        {
                        if ((buf1[x] != pbuf1[x]) || (buf2[x] != pbuf2[x]))     //      ( )
                                {
                                for (x = 0; x < 16; x++)                        //    
                                        {
                                        pbuf1[x] = buf1[x];
                                        pbuf2[x] = buf2[x];
                                        }
                                lcd_clear();                                    //    
                                lcd_gotoxy(0, 0);
                                lcd_puts(buf1);
                                lcd_gotoxy(0, 1);
                                lcd_puts(buf2);
                                break;
                                }
                        }
                address = command = 0;
                }
        else                                                    //   
                {
                if (!power_cmplt)                               //   ,    
                        {
                        i2c_start ();                           // 
                        i2c_write (tda_addr);
                        i2c_write (sub_address[volume]);          
                        i2c_write (56);                
                        i2c_stop ();
                        lcd_clear();
                        blight = dis;                           // 
                        power_cmplt = 1;                        // ,    
                        }
                }      
        }
}
